@@ -1,15 +1,13 @@
 /* public domain rewrite of strtol(3) */
 
+#include "ruby/missing.h"
 #include <ctype.h>
 
 long
-strtol(nptr, endptr, base)
-    char *nptr;
-    char **endptr;
-    int base;
+strtol(const char *nptr, char **endptr, int base)
 {
     long result;
-    char *p = nptr;
+    const char *p = nptr;
 
     while (isspace(*p)) {
 	p++;
@@ -23,7 +21,7 @@ strtol(nptr, endptr, base)
 	result = strtoul(p, endptr, base);
     }
     if (endptr != 0 && *endptr == p) {
-	*endptr = nptr;
+	*endptr = (char *)nptr;
     }
     return result;
 }
